use Test::More;
use Readonly;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::PerlTools qw (IsA);
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Core::API::File qw( GetFileContent );
use DE_EPAGES::AvaTax::API::Constants qw ( 
    AVATAX_TEXTCASE_Upper
    AVATAX_TEXTCASE_Default
    AVATAX_RESULTCODE_Error
);
use DE_EPAGES::AvaTax::API::Object::RemoteAddressSvc;
use DE_EPAGES::AvaTax::API::DTO::ValidateRequest;
use DE_EPAGES::AvaTax::API::DTO::Address;

use strict;

Readonly::Scalar my $TEST_LOCAL => 0;

sub testPing {
    my ($Remote) = @_;

    ok($Remote->logging(), 'logging enabled');
    my $Response;
    if ($TEST_LOCAL) {
        $Response = DE_EPAGES::AvaTax::API::XML::PingResponseHandler->importXML(
            ${GetFileContent('PingResponse.xml')}
        );
    } else {
        $Response = $Remote->ping();
    }

    ok(IsA($Response, 'DE_EPAGES::AvaTax::API::DTO::PingResult'), 'Response is a PingResult');
    cmp_ok($Response->TransactionId,'>',0,'TransactionID: '.$Response->TransactionId);
    is($Response->ResultCode, 'Success', 'ResultCode: Success');
    ok($Response->Version =~ /^[0-9.]+$/, 'Response version: '.$Response->Version);
}

sub testIsAuthorized {
    my ($Remote) = @_;
    my $Operations = 'Validate';
    my $Response;

    if ($TEST_LOCAL) {
        $Response = DE_EPAGES::AvaTax::API::XML::IsAuthorizedResponseHandler->importXML(
            ${GetFileContent('IsAuthorizedAddressResponse.xml')}
        );
    } else {
        $Response = $Remote->isAuthorized($Operations);
    }

    ok(IsA($Response, 'DE_EPAGES::AvaTax::API::DTO::IsAuthorizedResult'), 'Response is a IsAuthorizedResult');
    cmp_ok($Response->TransactionId,'>',0,'TransactionID: '.$Response->TransactionId);
    is($Response->ResultCode, 'Success', 'ResultCode: Success');
    is($Response->Operations, $Operations, 'Operations: '.$Operations);
    ok($Response->Expires =~ /^\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}$/x, 'Expires: '.$Response->Expires);
}

sub testValidate {
    my ($Remote) = @_;

    my $Request = DE_EPAGES::AvaTax::API::DTO::ValidateRequest->new();
    my $Address = DE_EPAGES::AvaTax::API::DTO::Address->new();
    $Address->Line1('900 Winslow Way');
    $Address->Line2('Suite 130');
    $Address->City('Bainbridge Is');
    $Address->Region('WA');
    $Address->PostalCode('98110-2450');

    $Request->pushAddress($Address);
    $Request->Taxability('true');
    $Request->TextCase(AVATAX_TEXTCASE_Upper);

    my $Response;
    if ($TEST_LOCAL) {
        $Response = DE_EPAGES::AvaTax::API::XML::ValidateResponseHandler->importXML(
            ${GetFileContent('ValidateResponse.xml')}
        );
    } else {
        $Response = $Remote->validate($Request);
    }

    ok(IsA($Response, 'DE_EPAGES::AvaTax::API::DTO::ValidateResult'), 'Response is a ValidateResult');
    cmp_ok(scalar @{$Response->ValidAddressList}, '==',1, 'number of valid addresses');

    my $ValidAddress = $Response->popValidAddress;
    is($ValidAddress->Line1, '900 WINSLOW WAY E STE 130','Line1');
    is($ValidAddress->Line4, 'BAINBRIDGE IS WA 98110-2450','Line4');
    is($ValidAddress->FipsCode, '5303500000','FipsCode');
    is($ValidAddress->County, 'KITSAP','County');
    is($Response->Taxable, 'true','Taxable');

}

sub testValidateEmptyAddress {
    my ($Remote) = @_;
    my $Request = DE_EPAGES::AvaTax::API::DTO::ValidateRequest->new();
    my $Address = DE_EPAGES::AvaTax::API::DTO::Address->new();
    $Request->pushAddress($Address);
    my $Response;

    if ($TEST_LOCAL) {
        $Response = DE_EPAGES::AvaTax::API::XML::ValidateResponseHandler->importXML(
            ${GetFileContent('ValidateEmptyAddressResponse.xml')}
        );
    } else {
        $Response = $Remote->validate($Request);
    }

    is($Response->ResultCode, AVATAX_RESULTCODE_Error, 'result is error.');
    cmp_ok(scalar @{$Response->MessageList}, '>', 0, 'error messages exist.');
    my $MessageAttributes = $Response->popMessage->Attributes;
    is($MessageAttributes->{'Name'}, 'InsufficientAddressError', 'message is InsufficientAddressError');

}

sub testValidateInvalidZIP {
    my ($Remote) = @_;
    my $Request = DE_EPAGES::AvaTax::API::DTO::ValidateRequest->new();
    my $Address = DE_EPAGES::AvaTax::API::DTO::Address->new();
    $Address->Line1('900 Winslow Way');
    $Address->Line2('Suite 130');
    $Address->City('Bainbridge Is');
    $Address->Region('GA');
    $Address->PostalCode('00000');
    $Request->pushAddress($Address);
    $Request->TextCase(AVATAX_TEXTCASE_Upper);

    my $Response;
    if ($TEST_LOCAL) {
        $Response = DE_EPAGES::AvaTax::API::XML::ValidateResponseHandler->importXML(
            ${GetFileContent('ValidateInvalidZIPResponse.xml')}
        );
    } else {
        $Response = $Remote->validate($Request);
    }

    is($Response->ResultCode, AVATAX_RESULTCODE_Error, 'result is error.');
    cmp_ok(scalar @{$Response->MessageList}, '>', 0, 'error messages exist.');
    my $MessageAttributes = $Response->popMessage->Attributes;
    is($MessageAttributes->{'Name'}, 'PostalCodeError', 'message is PostalCodeError');

}


sub testValidateInvalidStreet {
    my ($Remote) = @_;
    my $Request = DE_EPAGES::AvaTax::API::DTO::ValidateRequest->new();
    my $Address = DE_EPAGES::AvaTax::API::DTO::Address->new();
    $Address->Line1('999 Hallo Test');
    $Address->City('Bainbridge Is');
    $Address->Region('GA');
    $Address->PostalCode('98110');
    $Request->pushAddress($Address);
    $Request->TextCase(AVATAX_TEXTCASE_Default);

    my $Response;
    if ($TEST_LOCAL) {
        $Response = DE_EPAGES::AvaTax::API::XML::ValidateResponseHandler->importXML(
            ${GetFileContent('ValidateInvalidStreetResponse.xml')}
        );
    } else {
        $Response = $Remote->validate($Request);
    }

    is($Response->ResultCode, AVATAX_RESULTCODE_Error, 'result is error.');
    cmp_ok(scalar @{$Response->MessageList}, '>', 0, 'error messages exist.');
    my $MessageAttributes = $Response->popMessage->Attributes;
    is($MessageAttributes->{'Name'}, 'AddressUnknownStreetError', 'message is StreetError');

}

RunScript(
    'Sub' => sub {

        my $Remote = DE_EPAGES::AvaTax::API::Object::RemoteAddressSvc->new(
            ServiceURL => 'https://development.avalara.net/Address/AddressSvc.asmx',
            Username   => '1100037422',
            Password   => 'A0FF1C77C0570F29',
            logging    => 1,
        );

        eval {
            testPing($Remote);
            testIsAuthorized($Remote);
            testValidate($Remote);
            testValidateEmptyAddress($Remote);
            testValidateInvalidZIP($Remote);
            testValidateInvalidStreet($Remote);
        };
        if (ExistsError()) {
            my $Error = GetError();
            fail('ERROR: '. substr($Error->shortMessage, 0, 4096) . "\n");
        }
        done_testing();
    }
);

