use Test::More;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::File qw( GetFileContent );
use DE_EPAGES::Core::API::PerlTools qw (GetCurrentDateTime);
use DE_EPAGES::AvaTax::API::Object::RemoteTaxSvc;
use DE_EPAGES::AvaTax::API::DTO::GetTaxRequest;
use DE_EPAGES::AvaTax::API::DTO::BaseAddress;
use DE_EPAGES::AvaTax::API::DTO::Line;
use DE_EPAGES::AvaTax::API::XML::IsAuthorizedResponseHandler;
use DE_EPAGES::AvaTax::API::Constants qw (
    AVATAX_DETAILLEVEL_Tax
    AVATAX_RESULTCODE_Success
);
use strict;

# switch to 1 for local testing purposes.
my $TestLocal = 0;

sub testIsAuthorized {
    my ($Remote) = @_;
    $Remote->logging(1);
    my $Operations = 'GetTax';

    my $Response;
    if ($TestLocal) {
        $Response = DE_EPAGES::AvaTax::API::XML::IsAuthorizedResponseHandler->importXML(
            ${GetFileContent('IsAuthorizedTaxResponse.xml')}
        );
    } else {
        $Response = $Remote->isAuthorized($Operations);
    }

    isa_ok($Response, 'DE_EPAGES::AvaTax::API::DTO::IsAuthorizedResult');
    cmp_ok($Response->TransactionId,'>',0,'TransactionID: '.$Response->TransactionId);
    is($Response->ResultCode, 'Success', 'ResultCode: Success');
    is($Response->Operations, $Operations, 'Operations: '.$Operations);
    ok($Response->Expires =~ /^\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}$/x, 'Expires: '.$Response->Expires);
}

sub testPing {
    my ($Remote) = @_;
    $Remote->logging(1);

    my $Response;
    if ($TestLocal) {
        $Response = DE_EPAGES::AvaTax::API::XML::PingResponseHandler->importXML(
            ${GetFileContent('PingResponse.xml')}
        );
    } else {
        $Response = $Remote->ping();
    }

    isa_ok($Response, 'DE_EPAGES::AvaTax::API::DTO::PingResult');
    cmp_ok($Response->TransactionId,'>',0,'TransactionID '.$Response->TransactionId);
    is($Response->ResultCode, 'Success', 'ResultCode: Success');
    ok($Response->Version =~ /^[0-9.]+$/, 'Response version: '.$Response->Version);
}

sub testPingFailure {
    my ($Remote) = @_;
    local $Remote->{'Username'} = 'DummyTest';
    my $Response;

    if ($TestLocal) {
        $Response = DE_EPAGES::AvaTax::API::XML::PingResponseHandler->importXML(
            ${GetFileContent('PingFailure.xml')}
        );
    } else {
        $Response = $Remote->ping();
    }

    isa_ok($Response, 'DE_EPAGES::AvaTax::API::DTO::SOAPFault');
    is($Response->faultcode,'s:Client','FailureTest - FaultCode: '.$Response->faultcode);
    ok(length( $Response->faultstring )> 0, 'FailureTest - Faultstring: '.$Response->faultstring);
}

sub testGetTax {
    my ($Remote) = @_;
    my $Request = getTaxRequest1();

    my $Response;
    if ($TestLocal) {
        $Response = DE_EPAGES::AvaTax::API::XML::GetTaxResponseHandler->importXML(
            ${GetFileContent('GetTaxResponse.xml')}
        );
    } else {
        $Response = $Remote->getTax($Request);
    }
    isa_ok($Response, 'DE_EPAGES::AvaTax::API::DTO::GetTaxResult');
    is($Response->ResultCode(), AVATAX_RESULTCODE_Success, 'ResultCode: Success');
    is(scalar @{$Response->TaxLineList()}, scalar @{$Request->LineList()}, 'TaxLineList size');
    my $TaxLine = shift @{$Response->TaxLineList()};
    is($TaxLine->No(),1, 'TaxLine.No: 1');
    is($TaxLine->TaxCode(),'P0000000', 'TaxLine.TaxCode: P0000000');
    is($TaxLine->Taxability(), 'true', 'TaxLine.Taxability: true');
    is($TaxLine->BoundaryLevel(), 'Address', 'TaxLine.BoundaryLevel: Address');
    is($TaxLine->Exemption(), 0, 'TaxLine.Exemption: 0');
    is($TaxLine->Discount(),0, 'TaxLine.Discount: 0');
    is($TaxLine->Taxable(), 1000, 'TaxLine.Taxable: 1000');
    cmp_ok($TaxLine->Rate(), '==', 0.095, 'TaxLine.Rate: 0.095');
    cmp_ok($TaxLine->Tax(), '==',95, 'TaxLine.Tax: 95');
    cmp_ok($TaxLine->TaxCalculated(), '==',95, 'TaxLine.TaxCalculated: 95');
    is($TaxLine->TaxIncluded(), 'false','TaxLine.TaxIncluded');

    is(scalar @{$TaxLine->TaxDetailList()}, 2, 'Existing TaxDetails');
    my $TaxDetail = shift @{$TaxLine->TaxDetailList()};
    is($TaxDetail->JurisType(), 'State', 'Jurisdiction Type: State');
    is($TaxDetail->JurisCode(), '53', 'JurisCode: 53');
    is($TaxDetail->Taxable(), 1000, 'Taxable: 1000');
    cmp_ok($TaxDetail->Rate(), '==', 0.065, 'Rate: 0.065');
    is($TaxDetail->RateType(),'G', 'RateType: G');
    cmp_ok($TaxDetail->Tax(), '==', 65, 'Tax: 65');
    is($TaxDetail->NonTaxable(), 0, 'NonTaxable: 0');
}

sub getTaxRequest1 {
    my $CurDate = GetCurrentDateTime->strftime('%Y-%m-%d');
    my $Request = DE_EPAGES::AvaTax::API::DTO::GetTaxRequest->new;
    $Request->CompanyCode('EPAGE');
    $Request->OriginCode('Origin');
    $Request->DestinationCode('Dest');
    $Request->DetailLevel(AVATAX_DETAILLEVEL_Tax);
    my $Origin = DE_EPAGES::AvaTax::API::DTO::BaseAddress->new;
    $Origin->AddressCode('Origin');
    $Origin->Line1('Avalara');
    $Origin->Line2('900 Winslow Way');
    $Origin->Line3('Suite 100');
    $Origin->City('Bainbridge Island');
    $Origin->Region('WA');
    $Origin->PostalCode('98110');
    $Origin->Country('USA');
    $Request->pushAddress($Origin);

    my $Dest = DE_EPAGES::AvaTax::API::DTO::BaseAddress->new;
    $Dest->AddressCode('Dest');
    $Dest->Line1('3130 Elliott');
    $Dest->City('Seattle');
    $Dest->Region('WA');
    $Dest->PostalCode('98121');
    $Dest->Country('USA');
    $Request->pushAddress($Dest);

    #product
    my $Line = DE_EPAGES::AvaTax::API::DTO::Line->new;
    $Line->No(1);
    $Line->ItemCode('Item0001');
    # 256 chars.
    $Line->Description('1234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456');
    $Line->Qty(1);
    $Line->Amount(1000);
    $Request->pushLine($Line);

    #Shipping.
    $Line = DE_EPAGES::AvaTax::API::DTO::Line->new;
    $Line->No(2);
    $Line->ItemCode('FR');
    $Line->Qty(1);
    $Line->Amount(10.00);
    $Request->pushLine($Line);

    $Request->PaymentDate($CurDate);
    return $Request;
}

RunScript(
    'Sub' => sub {

        my $Remote = DE_EPAGES::AvaTax::API::Object::RemoteTaxSvc->new(
            ServiceURL => 'https://development.avalara.net/Tax/TaxSvc.asmx',
            Username   => '1100037422',
            Password   => 'A0FF1C77C0570F29',
        );

        testPing($Remote);
        testIsAuthorized($Remote,);
        testPingFailure($Remote);
        testGetTax($Remote);
        done_testing();
    }
);
