use strict;
use Test::More;
use Time::HiRes qw(time);
use utf8;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::DataEncryption qw(Decrypt Encrypt IsEncrypted );
use DE_EPAGES::Database::API::Connection qw( GetCurrentDBHandle RunOnStore GetTestSite );

# setup database; usually each db section should contain an "key" entry where the aes can be found, like:
#
#   [Site.Default]
#   datasource=dbi:mysql:mysql_socket=/var/lib/mysql/mysql.sock;database=storedb
#   login=usr_storedb
#   passwd=epages
#   active=1
#   key=hst58a42hvbt8671loegysq87n4mc7z1
#
# if not, we add one key here.

local $ENV{'EPAGES_TESTSITE'} = 'Store' unless $ENV{'EPAGES_TESTSITE'};
RunScript( Sub => sub {
    RunOnStore( Store => GetTestSite(), Sub => sub {

        my $dbi = GetCurrentDBHandle();
        $dbi->{'key'} = 'hst58a42hvbt8671loegysq87n4mc7z1' unless $dbi->{'key'};

        # ------------------------------------------------------
        # first test: simple string
        # ------------------------------------------------------
        my $data = 'Hallo, das ist ein Test';

        my $encrypted = Encrypt($data);
        ok( not ($data eq $encrypted), 'encoded data is NOT the same like original' );

        my $plaintext = Decrypt($encrypted);
        ok( $data eq $plaintext, 'decoded encoded data is the same like original' );

        # ------------------------------------------------------
        # second test:  string with special characters
        # ------------------------------------------------------
        $data = 'Hallöle äüöß 123€';

        $encrypted = Encrypt($data);
        print "encrypted: $encrypted\n";
        ok( not ($data eq $encrypted), 'encoded data is NOT the same like original' );

        $plaintext = Decrypt($encrypted);
        ok( $data eq $plaintext, 'decoded encoded data is the same like original' );

        # ------------------------------------------------------
        # third test:  very long string
        # ------------------------------------------------------
        $data =<<'EOT';
Lorem ipsum dolor sit amet, consetetur sadipscing elitr,
sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat,
sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum.
Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet.
Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor
invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam
et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet.';
EOT
        $encrypted = Encrypt($data);
        print "encrypted: $encrypted\n";
        ok( not ($data eq $encrypted), 'encoded data is NOT the same like original' );

        $plaintext = Decrypt($encrypted);
        ok( $data eq $plaintext, 'decoded encoded data is the same like original' );

        # ------------------------------------------------------
        # fourth test:  no encryption (downward compartibility)
        # ------------------------------------------------------
        $data = 'Hey, nicht verschlüsseln!';
        $dbi->{'key'} = '';

        $encrypted = Encrypt($data);
        print "encrypted: $encrypted\n";
        ok( $data eq $encrypted, 'encoded data is the same like original' );

        $plaintext = Decrypt($encrypted);
        ok( $data eq $plaintext, 'decoded encoded data is the same like original' );


        # ------------------------------------------------------
        # fifth test:  performance
        # ------------------------------------------------------
        $data = 'Hallo, das ist ein Test';
        $dbi->{'key'} = 'hst58a42hvbt8671loegysq87n4mc7z1';
        use constant NUM_RUNS => 1000;

        my $t0 = time;
        foreach (1..NUM_RUNS) {
            Encrypt($data);
        }
        my $elapsed =  (time - $t0)/NUM_RUNS;
        ok( 1, "Needed $elapsed seconds per encode.");


        # ------------------------------------------------------
        # sixth test:  use Decrypt() on non encrypted data
        # ------------------------------------------------------
        $data = 'Hallo, das ist ein Test';
        $plaintext = Decrypt($data);
        is( $plaintext, $data, 'Workaround for using Decrypt() on plain text. (EPG-19097)');

        # use IsEncrypted() on encrypted and non encrypted data - with key
        ok( IsEncrypted(Encrypt($data)), 'IsEncrypted("encrypted text") - with key -> true');
        ok( !IsEncrypted($data), 'IsEncrypted("plain text") - with key -> false');

        # use IsEncrypted() on encrypted and non encrypted data - without key
        $dbi->{'key'} = '';
        ok( !IsEncrypted(Encrypt($data)), 'IsEncrypted("encrypted text") - without key -> false');
        ok( !IsEncrypted($data), 'IsEncrypted("plain text") - without key -> false');

        # ------------------------------------------------------
        # seventh test:  test length of encrypted data
        # ------------------------------------------------------
        $dbi->{'key'} = 'hst58a42hvbt8671loegysq87n4mc7z1' unless $dbi->{'key'};
        my $NumChars = length Encrypt('1111222233334444');
        ok( $NumChars == 64, "Encrypted data length == $NumChars" );

        done_testing();
    },);
});