#!/bin/env bash

PWD=`pwd`

usage() {
  echo "Usage:" 1>&2
  echo "  EPAGES_PROJECT=epages6 $0 -i INITIAL_VERSION -p PATCH_VERSION -b BACKUP_DIR -P EPAGES_PROJECT [-R]" 1>&2
  echo "Options:" 1>&2
  echo "   EPAGES_PROJECT: envvar of the epages project, usually epages6" 1>&2
  echo "   -i: initial version (integer < 10^19)" 1>&2
  echo "   -p: patch version (dotted)" 1>&2
  echo "   -b: backup directory" 1>&2
  echo "   -R: restore" 1>&2
  exit 1
}

unset INITIAL_VERSION NEW_PATCH_VERSION BACKUP_DIR RESTORE EPAGES_PROJECT

while getopts i:p:b:RP: c ; do
  case $c in
    i) INITIAL_VERSION=$OPTARG ;;
    p) NEW_PATCH_VERSION=$OPTARG ;;
    b) BACKUP_DIR=$OPTARG ;;
    R) RESTORE=$c ;;
    P) EPAGES_PROJECT=$OPTARG ;;
    *) echo "$0: Unknown option: $c" ; usage ;;
  esac
done
shift `expr $OPTIND - 1`

if [ ! -w /etc/passwd ] ; then
  echo "$0: only 'root' can execute this script." 1>&2
  exit 1
fi

[ -z "$EPAGES_PROJECT" ] && { echo "$0: Option required: -P" ; usage; }
[ -z "$NEW_PATCH_VERSION" ] && { echo "$0: Option required: -p" ; usage; }
[ -z "$INITIAL_VERSION" ] && { echo "$0: Option required: -i" ; usage; }
[ -n "${INITIAL_VERSION//[0-9]/}" ] && { echo "$0: Not a number: -i $INITIAL_VERSION" ; usage; }
INITIAL_VERSION=`expr $INITIAL_VERSION + 0 2>/dev/null`
[ ${#INITIAL_VERSION} -gt 18 ] && { echo "$0: Number must no exeed 10^19: -i $INITIAL_VERSION" ; usage; }

# set environment

export EPAGES_PROJECT
EP_DEFAULT=/etc/default/"$EPAGES_PROJECT"

if grep -q "^PATCH_VERSION=$NEW_PATCH_VERSION$" "$EP_DEFAULT" ; then
  [ -z "$RESTORE" ] && {
    echo "INFO: $BN skipped, because '$EP_DEFAULT' already patched" 1>&2
    exit 0
  }
elif [ -n "$RESTORE" ] ; then
  echo "INFO: $BN skipped, because '$EP_DEFAULT' not patched" 1>&2
  exit 0
fi

[ -n "$BACKUP_DIR" -a ! -d "$BACKUP_DIR" ] && { mkdir -p "$BACKUP_DIR" || exit 1; }

. "$EP_DEFAULT"

EPAGES_ETC=$(dirname "$EPAGES")/etc

# restore epages scripts

if [ -n "$RESTORE" ] ; then
  if [ -n "$BACKUP_DIR" ] ; then
     for i in default/"$EPAGES_PROJECT" init.d/"$EPAGES_PROJECT" init.d/epages6su ; do
      [ -f "$BACKUP_DIR"/$i ] || continue
      cat "$BACKUP_DIR"/$i > /etc/$i
    done
    /etc/init.d/"$EPAGES_PROJECT" perm admin
  fi
  exit 0
fi

# backup epages scripts

if [ -n "$BACKUP_DIR" ] ; then
  for i in default/"$EPAGES_PROJECT" init.d/"$EPAGES_PROJECT" init.d/epages6su ; do
    [ -f /etc/$i ] || continue
    mkdir -p -m 755 "$BACKUP_DIR"/${i%/*}
    cat /etc/$i > "$BACKUP_DIR"/$i
  done
fi

# edit yum.repos.d/epages.repo

INI_CONFIG="$EPAGES"/bin/ini-config.sh

if $(ls /etc/yum.repos.d/epages*.repo >/dev/null 2>&1) ; then
  PREFIX=$(rpm -q epages-common --qf '[%{INSTPREFIXES}\n]' | head -1)
  if [ -n "$PREFIX" -a "$PREFIX" != /srv/epages -a ! -d /srv/epages/eproot/bin ] ; then
    rmdir /srv/epages/* 2>/dev/null
    rm -f /srv/epages/* 2>/dev/null
    rmdir /srv/epages 2>/dev/null
    rm -f /srv/epages 2>/dev/null
    mkdir -m 755 -p /srv
    if [ -e /srv/epages -a ! -e /srv/epages/eproot ] ; then
      ( cd /srv/epages ; ln -s "$PREFIX"/eproot )
    elif [ ! -e /srv/epages ] ; then
      ( cd /srv ; ln -s "$PREFIX" epages )
    fi
  fi

  VRS=$(rpm -q epages-common --qf '%{version}' | awk -F. '/^[0-9.]+$/{printf "%s.%s", $1, $2}')
  if [ -n "$VRS" ] ; then
    for j in /etc/yum.repos.d/*epages* ; do
      sed -i "s,/RPMS/6\.[0-9]\+\([^0-9.]\),/RPMS/$VRS\1," $j
    done
  fi
fi

# edit sysconfig/epages6

chmod 644 /etc/sysconfig/epages6 2>/dev/null

# edit init.d/epages6

cd "$EPAGES_ETC" || exit 1

echo "$EPAGES_PROJECT" > epages-project
sed "s,_EPAGES_PROJECT_,$EPAGES_PROJECT,g" init.d/epages6.ept > init.d/epages6
chmod 755 init.d/epages6

# edit default/epages6

if [ $INITIAL_VERSION -lt 6011000000 -a -f "$BACKUP_DIR"/default/"$EPAGES_PROJECT" ] ; then

  grep -v ' chcon ' "$BACKUP_DIR"/default/"$EPAGES_PROJECT" \
  | sed 's,\(ls -d [^ ]*\) |,\1 2>/dev/null |,;s%.*MANPATH=.*%MP=$(echo $PATH | $AWK '"'"'BEGIN{RS=":";ORS=":"}/\\/bin/{sub(/\\/bin/,"/man");print}'"'"')\nMANPATH=`pathmunge "$MANPATH:$MP:$EPAGES_PERL/man:$HTTPD_ROOT/man"`%' \
  > default/epages6

  echo '#
#       selinux
#       -------
#
EPAGES_ENABLE_SELINUX=0
EPAGES_ENABLE_PERL_CGI=1
export EPAGES_ENABLE_SELINUX EPAGES_ENABLE_PERL_CGI
#
if [[ ${EPAGES_ENABLE_SELINUX:-0} != 0 ]] ; then
  [[ ${EPAGES_ENABLE_PERL_CGI:-0} = 0 ]] && PERL_CGI=--perl-cgi || unset PERL_CGI
  $EPAGES/bin/se-file-context.sh --set $PERL_CGI
fi
#' >> default/epages6
  echo "PATCH_VERSION=$NEW_PATCH_VERSION" >> default/epages6

fi

# link epages scripts

for i in default/"$EPAGES_PROJECT" init.d/"$EPAGES_PROJECT" init.d/epages6su ; do
  [ -f /etc/$i ] || continue
  rm -f /etc/$i
  if [ ${i#*/} = epages6su ] ; then
    ln -s "$EPAGES_ETC"/$i /etc/$i
  else
    ln -s "$EPAGES_ETC"/${i%/*}/epages6 /etc/$i
  fi
done

# set access rights

chmod 755 /etc/default/"$EPAGES_PROJECT"
chmod 740 /etc/init.d/"$EPAGES_PROJECT"
/etc/init.d/"$EPAGES_PROJECT" perm admin
